/******************************************************************************
*                                                                             *
*   t i c o n s t                                                             *
*                                                                             *
*   by Stefan Heule                                                           *
*   http://stefanheule.com                                                    *
*                                                                             *
*   Source Code                                                               *
*                                                                             *
*******************************************************************************

ticonst - replacement for the standard units menu
Copyright (C) 2009-2010 Stefan Heule

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/



// ----------------------------------------------------------------- //
// --- header files
// ----------------------------------------------------------------- //
#include "header.h"


// ----------------------------------------------------------------- //
// --- globals
// ----------------------------------------------------------------- //
void* screen;                // backbuffer
CUST_FONT myfont;
OPTIONS pref;                // the preferences

#ifdef USE_TI89
	#define lm_max 8 // max number of items in the left menu
	#define rm_max 8 // max number of items in the rithg menu
#else
	#define lm_max 10
	#define rm_max 10
#endif

#define line_height 9 // height of one line in pixel

// ----------------------------------------------------------------- //
// --- main
// ----------------------------------------------------------------- //
void _main(void)
{
	short i,j;
	short key;
	char* ctmp;
	const char* c;
	
  pref.lm_up = 0; // index of first group that is displayed
	pref.lm_mid = 0; // cursor pos
	pref.rm_up = 0;
	pref.rm_mid = 0;
	pref.save_pos = 0;
	pref.cur = 0; // 0 = left menu, 1 = right menu
	pref.mode = 0; // 0 = name, 1 = value
	
	// load the charlist (loadBackup will output an error msg)
	if (loadBackup() == FALSE)
		return;
	
	const short xs = 4; // left border
	short xxs = xs+130; // group / unit separation line
#ifdef USE_TI89
	const short xe = 159-xs; // right border
#else
	const short xe = 239-xs; // right border
#endif
	const short ys = 4; // top border
	const short ye = ys+13+lm_max*line_height; // bottom border
	
	HANDLE *h = NULL; // handle buffer
	short hc = 0; // count of h
	char buf[50];
	
	const short rsb_max = ye-ys-11; // num pixels for the scrollbar
#ifdef USE_TI89
	short rsb_height = ((-9*nlst[pref.lm_mid+pref.lm_up])>>1)+104; // scrollbar height
#else
	short rsb_height = ((-11*nlst[pref.lm_mid+pref.lm_up])>>1)+126; // scrollbar height
#endif
	short rsb_pos; // scrollbar position
	short rsb; // is there a scrollbar?
	const short lsb_max = ye-ys-11; // num pixels for the scrollbar
	short lsb_height = 15; // scrollbar height
	short lsb_pos = ys+9; // scrollbar position
	
	EVENT ev;
	short send_ev = 0;
	
	short repeat = 1; // autorepeat
	
	// allocate some space for our virtual screen (backbuffer)
	if ((screen = malloc(2*LCD_SIZE)) == NULL) {
		ST_helpMsg(MSG_ERROR_MEMORY);
		return;
	}
	
	// fast font loading
	if (!InitializeFastString()) {
		free(screen);
		ST_helpMsg(MSG_ERROR_MEMORY);
		return;
	}
	
	// backup orginal screen
	memcpy(screen+LCD_SIZE, LCD_MEM, LCD_SIZE);
	
	// set the graphic port to our virtual screen for backbuffering
	PortSet(screen, 239, 127);
	
	// main loop
	while (1)
	{

#ifdef USE_TI89
		if (pref.cur == 0) xxs = xs+100;
		else xxs = xs+46;
#endif

		// main border and background
		//memset(screen, 0, LCD_SIZE);
		memcpy(screen, screen+LCD_SIZE, LCD_SIZE);
		
		// titlebar and title
		DrawDialog(xs, ys, xe, ye, MSG_ABOUT_PROGRAM_NAME" "MSG_ABOUT_VERSION, TRUE);
		
		// group separation
		DrawLine(xxs,ys+9, xxs,ye-3, A_NORMAL);
		
		// groups
		for (i = 0; i < lm_max && i+pref.lm_up < ngroups; i++) {
#ifdef USE_TI89
			if (pref.cur == 1) {
				j = 0;
				while (groups[i+pref.lm_up].name[j] != 0 && j < 6) {
					buf[j] = groups[i+pref.lm_up].name[j]; j++;
				}
				buf[j] = 0;
				FastDrawString6x8(xs+5, ys+11+i*line_height, buf, A_XOR);
			} else {
				FastDrawString6x8(xs+5, ys+11+i*line_height, groups[i+pref.lm_up].name, A_XOR);
			}
#else
			FastDrawString6x8(xs+5, ys+11+i*line_height, groups[i+pref.lm_up].name, A_XOR);
#endif
		}
		
		// unit name
		for (i = 0; i < lm_max && i < (short)nlst[pref.lm_mid+pref.lm_up]; i++) {
			FastDrawString6x8(xxs+3, ys+11+i*line_height, groups[pref.lm_mid+pref.lm_up].lst[pref.rm_up+i].name, A_XOR);
		}
		
		// right scrollbar
		if (nlst[pref.lm_mid+pref.lm_up] > rm_max) rsb = 1;
		else rsb = 0;
		if (rsb) {
			rsb_pos = ys+9+(pref.rm_up*(rsb_max-rsb_height))/(nlst[pref.lm_mid+pref.lm_up]-rm_max);
			DrawLine(xe-4,ys+9, xe-4,ye-3, A_NORMAL);
			DrawLine(xe-3,rsb_pos, xe-3,rsb_pos+rsb_height-1, A_NORMAL);
		}
		// left scrollbar
		DrawLine(xxs-2,ys+9, xxs-2,ye-3, A_NORMAL);
		DrawLine(xxs-1,lsb_pos, xxs-1,lsb_pos+lsb_height-1, A_NORMAL);
		
		// cursor
		DrawRect(xs+4, ys+10+line_height*pref.lm_mid, xxs-4, ys+9+line_height*(pref.lm_mid+1), 1-pref.cur, 2*(1-pref.cur)+1);
		DrawRect(xxs+2, ys+10+line_height*pref.rm_mid, xe-4-rsb*2, ys+9+line_height*(pref.rm_mid+1), pref.cur, 2*(pref.cur)+1);
		
		// units desc
#ifdef USE_TI89
		if (pref.cur == 1) {
#endif
		for (i = 0; i < lm_max && i < (short)nlst[pref.lm_mid+pref.lm_up]; i++) {
			if (pref.mode == 0) {
				//DrawStr(xe-5-rsb*2-DrawStrWidth(groups[pref.lm_mid+pref.lm_up].lst[pref.rm_up+i].desc,F_4x6), ys+12+i*line_height, groups[pref.lm_mid+pref.lm_up].lst[pref.rm_up+i].desc, A_XOR);
				FastDrawString4x6(xe-5-rsb*2-Strlen(groups[pref.lm_mid+pref.lm_up].lst[pref.rm_up+i].desc), ys+12+i*line_height, groups[pref.lm_mid+pref.lm_up].lst[pref.rm_up+i].desc, A_XOR);
			}
			else {
				if (h == NULL) { // cache ready?
					hc = nlst[pref.lm_mid+pref.lm_up];
					h = malloc(sizeof(HANDLE)*hc);
					for (j = 0; j < hc; j++) h[j] = 0;
				}
				if (h[pref.rm_up+i] == 0) { // cache ready?
					sprintf(buf, "_%s", groups[pref.lm_mid+pref.lm_up].lst[pref.rm_up+i].name);
					NG_approxESI(SYMSTR(buf));
					h[pref.rm_up+i] = Parse1DExpr(top_estack, 0, 0);
					c = HeapDeref(h[pref.rm_up+i]);
					if ((ctmp = strchr(c, '*')) != NULL) *ctmp = 0;
					FastDrawString4x6(xe-5-rsb*2-Strlen(c), ys+12+i*line_height, c, A_XOR);
					switchBackbuffer();
					if (isKeyWaiting()) goto key;
				}
				else { // print cached value
					c = HeapDeref(h[pref.rm_up+i]);
					FastDrawString4x6(xe-5-rsb*2-Strlen(c), ys+12+i*line_height, c, A_XOR);
				}
			}
		}
#ifdef USE_TI89
		}
#endif
		
		// copy everything to the screen
		switchBackbuffer();
		
		key:
		// check for a keypress
		key = getKey(repeat);
		repeat = 1;
		
		// cursor right
		if (key == KEY_RIGHT) {
			pref.cur = 1;
		}
		
		// cursor left
		else if (key == KEY_LEFT) {
			pref.cur = 0;
		}
		
		// chg mode
		else if (key == KEY_MODE) {
			pref.mode = !pref.mode;
		}
		
		// cursor up
		else if (key == KEY_UP) {
			if (pref.cur == 0) {
				if (pref.lm_mid > 0) pref.lm_mid--;
				else if (pref.lm_up > 0) pref.lm_up--;
				else {pref.lm_up = ngroups - lm_max; pref.lm_mid = lm_max - 1;}
				
				// no autorepeat
				if (pref.lm_up+pref.lm_mid == 0) repeat = 0;
				
				// clear cache
				if (h) {
					for (i = 0; i < hc; i++) if (h[i]) HeapFree(h[i]);
					free(h); h = NULL;
				}
				// reset right curser
				pref.rm_up = pref.rm_mid = 0;
				
				// scrollbar height
#ifdef USE_TI89
				rsb_height = ((-9*nlst[pref.lm_mid+pref.lm_up])>>1)+104; // scrollbar height
#else
				rsb_height = ((-11*nlst[pref.lm_mid+pref.lm_up])>>1)+126; // scrollbar height
#endif
				
				// scrollbar pos
				lsb_pos = ys+9+(pref.lm_up*(lsb_max-lsb_height))/(ngroups-lm_max);
			}
			else {
				if (pref.rm_mid > 0) pref.rm_mid--;
				else if (pref.rm_up > 0) pref.rm_up--;
				else {
					if (nlst[pref.lm_mid+pref.lm_up] > rm_max) {pref.rm_up = nlst[pref.lm_mid+pref.lm_up] - rm_max; pref.rm_mid = rm_max - 1;}
					else {pref.rm_mid = nlst[pref.lm_mid+pref.lm_up]-1;}
				}
				
				// no autorepeat
				if (pref.rm_up+pref.rm_mid == 0) repeat = 0;
			}
		}
		
		// cursor down
		else if (key == KEY_DOWN) {
			if (pref.cur == 0) {
				if (pref.lm_mid < lm_max-1) pref.lm_mid++;
				else if (pref.lm_up < ngroups-lm_max) pref.lm_up++;
				else {pref.lm_up = pref.lm_mid = 0;}
				
				// no autorepeat
				if (pref.lm_up+pref.lm_mid == ngroups-1) repeat = 0;
				
				// clear cache
				if (h) {
					for (i = 0; i < hc; i++) if (h[i]) HeapFree(h[i]);
					free(h); h = NULL;
				}
				
				// reset right curser
				pref.rm_up = pref.rm_mid = 0;
				
				// scrollbar height
#ifdef USE_TI89
				rsb_height = ((-9*nlst[pref.lm_mid+pref.lm_up])>>1)+104; // scrollbar height
#else
				rsb_height = ((-11*nlst[pref.lm_mid+pref.lm_up])>>1)+126; // scrollbar height
#endif
				
				// scrollbar pos
				lsb_pos = ys+9+(pref.lm_up*(lsb_max-lsb_height))/(ngroups-lm_max);
			}
			else {
				if (pref.rm_mid < rm_max-1 && pref.rm_mid < nlst[pref.lm_mid+pref.lm_up]-1) pref.rm_mid++;
				else if (pref.rm_up+pref.rm_mid < nlst[pref.lm_mid+pref.lm_up]-1) pref.rm_up++;
				else {pref.rm_up = pref.rm_mid = 0;}
				
				// no autorepeat
				if (pref.rm_up+pref.rm_mid == nlst[pref.lm_mid+pref.lm_up]-1) repeat = 0;
			}
		}
		
		// paste the unit
		else if (key == KEY_ENTER) {
			
			// set the event to <paste>
			ev.Type = CM_PASTE_STRING;
			sprintf(buf, "_%s", groups[pref.lm_mid+pref.lm_up].lst[pref.rm_up+pref.rm_mid].name);
			ev.extra.pasteText = buf;
			
			// make sure the event is sent
			send_ev = 1;
			
			break;
		}
		
		// paste the unit
		else if (key == KEY_DIAMOND + KEY_ENTER) {
			
			// set the event to <paste>
			ev.Type = CM_PASTE_STRING;
			sprintf(buf, "z%s", groups[pref.lm_mid+pref.lm_up].lst[pref.rm_up+pref.rm_mid].name);
			ev.extra.pasteText = buf;
			
			// make sure the event is sent
			send_ev = 1;
			
			break;
		}
		
		// help
		else if (key == KEY_F1) {
			
			// white background
			DrawRect(xs+3, ys+9, xe-3, ye-3, 1, 0);
			
			FastDrawString4x6(xs+6, 17+0*8, MSG_HELP_ARROWS, A_XOR);
			FastDrawString4x6(xs+6, 17+1*8, "ENTER", A_XOR);
			FastDrawString4x6(xs+6, 17+2*8, "MODE", A_XOR);
			
			FastDrawString4x6(xs+6, 17+4*8, "F2", A_XOR);
			FastDrawString4x6(xs+6, 17+5*8, "F3", A_XOR);
			
			FastDrawString4x6(xs+38, 17+0*8, MSG_HELP_NAVIGATE, A_XOR);
			FastDrawString4x6(xs+38, 17+1*8, MSG_HELP_PASTE_UNIT, A_XOR);
			FastDrawString4x6(xs+38, 17+2*8, MSG_HELP_CHANGE_MODE, A_XOR);
			FastDrawString4x6(xs+38, 17+3*8, MSG_HELP_CHANGE_MODE_2, A_XOR);
			FastDrawString4x6(xs+38, 17+4*8, MSG_HELP_ABOUT, A_XOR);
			FastDrawString4x6(xs+38, 17+5*8, MSG_OPT_TITLE, A_XOR);
			
			// copy everything to the screen
			switchBackbuffer();
			
			if (getKey(1) == KEY_F2) key = KEY_F2;
		}
		
		// about
		if (key == KEY_F2) {
			
#ifdef USE_TI89
				#define start_offset_about 5
#else
				#define start_offset_about 10
#endif
			
			// white background
			DrawRect(xs+3, ys+9, xe-3, ye-3, 1, 0);
			
			FastDrawString6x8((xs+xe)/2-strlen(MSG_ABOUT_PROGRAM_NAME)*3,       start_offset_about+25, MSG_ABOUT_PROGRAM_NAME, A_XOR);
			FastDrawString4x6((xs+xe)/2-Strlen(MSG_ABOUT_VERSION)/2,            start_offset_about+35, MSG_ABOUT_VERSION, A_XOR);
			FastDrawString4x6((xs+xe)/2-Strlen(MSG_ABOUT_BY_AUTHOR)/2,          start_offset_about+50, MSG_ABOUT_BY_AUTHOR, A_XOR);
			FastDrawString4x6((xs+xe)/2-Strlen(MSG_ABOUT_BOOLSOFT_URL)/2,       start_offset_about+58, MSG_ABOUT_BOOLSOFT_URL, A_XOR);
			
			// copy everything to the screen
			switchBackbuffer();
			
			getKey(1);
		}
		
		
		// options
		else if (key == KEY_F3) {
			
			short backup = pref.save_pos;

#define options_width 60

			do
			{
				// draw everything
				
				// draw dialog
				DrawDialog(LCD_WIDTH/2 - options_width, 29, LCD_WIDTH/2 + options_width, 82, MSG_OPT_TITLE, FALSE);
				
				// label
				FastDrawString4x6(LCD_WIDTH/2 - options_width + 5, 44, MSG_OPT_SAVE_POSITION":", A_XOR);
				FastDrawString4x6(LCD_WIDTH/2 - options_width + 5, 56, MSG_OPT_HELP_1, A_XOR);
				FastDrawString4x6(LCD_WIDTH/2 - options_width + 5, 64, MSG_OPT_HELP_2, A_XOR);
				FastDrawString4x6(LCD_WIDTH/2 - options_width + 5, 72, MSG_OPT_HELP_3, A_XOR);
				
				// checkbox
				DrawRect(LCD_WIDTH/2 + options_width - 15, 42, LCD_WIDTH/2 + options_width - 15 + 9, 42+9, 0, 1);
				if (pref.save_pos) {
					Sprite8_XOR_R(LCD_WIDTH/2 + options_width - 15 + 1, 42 + 1, 8, checkmark_sprt, screen);
				}
				
				switchBackbuffer();
				
				// get a input
				key = getKey(FALSE);
				
				if (key == KEY_ENTER) {
					break;
				}
				else if (key == KEY_ESC) {
					pref.save_pos = backup;
					break;
				}
				else if (key == KEY_RIGHT || key == KEY_LEFT) {
					pref.save_pos = !pref.save_pos;
				}
			} while (TRUE);
			
			key = 0;
		}
		
		// exit
		else if (key == KEY_ESC) {
			break;
		}
	}
	
	// restore the graphic port
	PortRestore();
	
	// restore screen
	memcpy(LCD_MEM, screen+LCD_SIZE, LCD_SIZE);
	
	// free stuff
	free(screen);
	if (h) {
		for (i = 0; i < hc; i++) if (h[i]) HeapFree(h[i]);
		free(h); h = NULL;
	}
	FreeFonts();
	
	// save settings
	if (pref.save_pos) saveBackup();
	else deleteBackup();
	
	// erase the status line
	ST_eraseHelp();
	
	// send event if necessary
	if (send_ev) EV_sendEvent(AP_CURRENT, &ev);
}





































